import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.Toolkit.*;
import java.lang.*;

class Integrate extends Thread
	{
	private double h;
	public double[] y = new double[2];
	public double x;
	private int imethod;
	Chart parent;
	Graphics g;
	int var,npoints;
	Dimension dim;

	public Integrate(Chart refparent, int refnpoints)
		{
		y[0] = y[1] = 0;
		x = 0;
		h = 0.1;
		imethod = 0;
		g = refparent.getGraphics();
		var = refparent.var;
		npoints = refnpoints;
		dim = refparent.getSize();
		parent = refparent;
		}

	public void setIntegrateMethod(int setimethod)
		{
		imethod = setimethod;
		}

	public void setMeshlength(double seth)
		{
		h = seth;
		}

	public void NextPoint()
		{
		double h2 = h/2.0,h6 = h/6.0;
		double xh = x + h, xh2 = x + h2;
		double[] t1 = new double[2],t2 = new double[2],t3 = new double[2],k1 = new double[2],k2 = new double[2],k3 = new double[2],k4 = new double[2];
		int i;

		for(i=0;i<2;i++)
			{
			k1[i]=parent.f(x,y,i);
			t1[i]=y[i]+h2*k1[i];
			}
		for(i=0;i<2;i++)
			{
			k2[i]=parent.f(xh2,t1,i);
			t2[i]=y[i]+h2*k2[i];
			}
		for(i=0;i<2;i++)
			{
			k3[i]=parent.f(xh2,t2,i);
			t3[i]=y[i]+h*k3[i];
			}
		for(i=0;i<2;i++)
			{
			k4[i]=parent.f(xh,t3,i);
			y[i]+=h6*(k1[i]+2*k2[i]+2*k3[i]+k4[i]);
			}

		x += h;
		}

    public void run()
        {
        int i;
        int coordx,coordy;
	  double vmean = 0.0;

        for(i=0;i<=npoints;i++)
		{
		NextPoint();
		if(var == 1)
			vmean += y[1];
		coordx = (int)((x*(dim.width-6))/(npoints*h))+3;
		coordy = (int)((y[var] % (2*Math.PI))*(dim.height-6)/(2*Math.PI))+dim.height/2 ;
		g.drawLine(coordx,coordy,coordx,coordy);
		try { sleep(2); }
		catch(InterruptedException e) {}
		}

	  if(var == 1)
		{
		vmean /= npoints;
		coordy = (int)((vmean % (2*Math.PI))*(dim.height-6)/(2*Math.PI))+dim.height/2 ;
		g.setColor(Color.red);
		for(i=4;i<=dim.width-4;i++)
			g.drawLine(i,coordy,i,coordy);
		}
	  }

	}

class StaticImageComponent extends Canvas implements ImageObserver
	{
	String fileimg;
	Image img;
	int w,h;
	
	StaticImageComponent(String infileimg)
		{
		fileimg = infileimg;
		img = getToolkit().getImage(fileimg);		
		
		prepareImage(img,this);
		}

	StaticImageComponent(Image inimg)
		{
		img = inimg;
		
		prepareImage(inimg,this);
		}

	public boolean imageUpdate(Image img,int infoflags,int x,int y,int width,int height)
		{
		switch(infoflags)
			{
			case ImageObserver.ALLBITS:
				setSize(w = img.getWidth(this),h = img.getHeight(this));
				return(Boolean.FALSE.booleanValue());
			}
		return(Boolean.TRUE.booleanValue());
		}

	public void paint(Graphics g)
		{
		g.drawImage(img,0,0,this);
		}
	}



class KillFrame extends Frame implements ActionListener, ItemListener
	{
	Panel tabs,cards;
	Panel modeling,polar,trajectory;
	TextField R_tf;
	TextField C_tf;
	TextField Ic_tf;
	TextField I0_tf, I1_tf, nu1_tf, time_tf;
	Chart chart;
	Choice var;

	StaticImageComponent modeling_sic,polar_sic;

	double R,C,Ic,I0,I1,omega1,omegac,BetaC,Omega1;
	static double _2esuhbar = 3.0386E15; // 2e/h_bar in MKS	
	int npoints;

	public KillFrame()
		{
		setTitle("Josephson Junction Simulator");

		// Main
		GridBagLayout main_layout;
		setLayout(main_layout = new GridBagLayout());

		GridBagConstraints main_constraints = new GridBagConstraints();
	
		main_constraints.weightx = 10;
		main_constraints.weighty = 10;
		main_constraints.gridx   =   0;
		main_constraints.gridy   =   0;
		main_constraints.gridwidth  = 3;
		main_constraints.gridheight = 1;
		main_constraints.fill   = GridBagConstraints.NONE;
		main_constraints.anchor = GridBagConstraints.CENTER;

		Button _0button,_1button,_2button;

		tabs = new Panel();
		tabs.add(_0button = new Button("Modeling"));
		tabs.add(_1button = new Button("Polarization"));
//		tabs.add(new Button("Options"));
		tabs.add(_2button = new Button("Trajectory"));
//		tabs.add(new Button("IV diagram"));

		add(tabs);		
		main_layout.setConstraints(tabs,main_constraints);		

		_0button.addActionListener(this);
		_1button.addActionListener(this);
		_2button.addActionListener(this);

		cards = new Panel();
		cards.setLayout(new CardLayout());
		cards.add("Modeling", modeling = new Panel());
		cards.add("Polarization", polar = new Panel());
//		cards.add("Options", new Button("Prova"));
		cards.add("Trajectory", trajectory = new Panel());
//		cards.add("IC diagram", new Button("IV diagram"));
		add(cards);
		main_constraints.weightx = 100;
		main_constraints.weighty = 100;
		main_constraints.gridy   =   1;
		main_constraints.gridheight = 3;
		main_constraints.fill   = GridBagConstraints.BOTH;
		main_layout.setConstraints(cards,main_constraints);		
		
		// Modeling Pane
		Label R_label,C_label,Ic_label;
		GridBagLayout modeling_layout;
		modeling.setLayout(modeling_layout = new GridBagLayout());

		GridBagConstraints modeling_constraints = new GridBagConstraints();
		modeling_constraints.weightx = 0;
		modeling_constraints.weighty = 0;
		modeling_constraints.gridwidth  = 4;
		modeling_constraints.gridheight = 4;

		modeling_constraints.fill   = GridBagConstraints.HORIZONTAL;
		modeling_constraints.anchor = GridBagConstraints.CENTER;

		modeling_constraints.gridx   =   0;
		modeling_constraints.gridy   =   0;

		modeling_sic = new StaticImageComponent("rcsj.gif");
		modeling.add(modeling_sic);
		modeling_layout.setConstraints(modeling_sic,modeling_constraints);		

		modeling_constraints.gridwidth = 1;
		modeling_constraints.gridheight = 1;

		modeling_constraints.gridx = 5;
		modeling.add(R_label = new Label("Resistance [Ohm]",Label.CENTER));
		modeling_layout.setConstraints(R_label, modeling_constraints);
		modeling_constraints.gridy++;
		modeling.add(C_label = new Label("Capacity [pF]",Label.CENTER));
		modeling_layout.setConstraints(C_label, modeling_constraints);
		modeling_constraints.gridy++;
		modeling.add(Ic_label = new Label("Critical current [mA]",Label.CENTER));
		modeling_layout.setConstraints(Ic_label, modeling_constraints);

		R = 1;
		C = 5;
		Ic = 2;
		omegac = _2esuhbar*R*Ic*1E-12;
		BetaC = _2esuhbar*R*R*C*Ic*1E-15;

		modeling_constraints.fill   = GridBagConstraints.NONE;
		modeling_constraints.gridx   =   6;
		modeling_constraints.gridy   =   0;
		modeling.add(R_tf  = new TextField("1",5));
		modeling_layout.setConstraints(R_tf, modeling_constraints);
		modeling_constraints.gridy++;
		modeling.add(C_tf  = new TextField("5",5));
		modeling_layout.setConstraints(C_tf, modeling_constraints);
		modeling_constraints.gridy++;
		modeling.add(Ic_tf = new TextField("2",5));
		modeling_layout.setConstraints(Ic_tf, modeling_constraints);


		//Polarization Pane
		Label I0_label,I1_label,nu1_label;
		GridBagLayout polar_layout;
		polar.setLayout(polar_layout = new GridBagLayout());

		GridBagConstraints polar_constraints = new GridBagConstraints();
		polar_constraints.weightx = 0;
		polar_constraints.weighty = 0;
		polar_constraints.gridwidth  = 4;
		polar_constraints.gridheight = 4;

		polar_constraints.fill   = GridBagConstraints.HORIZONTAL;
		polar_constraints.anchor = GridBagConstraints.CENTER;

		polar_constraints.gridx   =   0;
		polar_constraints.gridy   =   0;

		polar_sic = new StaticImageComponent("polar.gif");
		polar.add(polar_sic);
		polar_layout.setConstraints(polar_sic,polar_constraints);		

		polar_constraints.gridwidth = 1;
		polar_constraints.gridheight = 1;

		polar_constraints.gridx   =   5;
		polar.add(I0_label = new Label("Bias current [mA]",Label.CENTER));
		polar_layout.setConstraints(I0_label, polar_constraints);
		polar_constraints.gridy++;
		polar.add(I1_label = new Label("RF current [mA]",Label.CENTER));
		polar_layout.setConstraints(I1_label, polar_constraints);
		polar_constraints.gridy++;
		polar.add(nu1_label = new Label("RF frequency [GHz]",Label.CENTER));
		polar_layout.setConstraints(nu1_label, polar_constraints);

		I0 = 0;
		I1 = 1;
		omega1 = 2*Math.PI*10; 
		Omega1 = omega1 / omegac;

		polar_constraints.fill   = GridBagConstraints.NONE;
		polar_constraints.gridx   =   6;
		polar_constraints.gridy   =	0;
		polar.add(I0_tf = new TextField("0",5));
		polar_layout.setConstraints(I0_tf, polar_constraints);
		polar_constraints.gridy++;
		polar.add(I1_tf = new TextField("1",5));
		polar_layout.setConstraints(I1_tf, polar_constraints);
		polar_constraints.gridy++;
		polar.add(nu1_tf = new TextField("10",5));
		polar_layout.setConstraints(nu1_tf, polar_constraints);

		// Trajectory Pane
		Panel trajectory_options;
		Button start_button;

		GridBagLayout graph_layout;
		trajectory.setLayout(graph_layout = new GridBagLayout());		

		GridBagConstraints graph_constraints = new GridBagConstraints();
		graph_constraints.weightx = 10;
		graph_constraints.weighty = 10;
		graph_constraints.gridx   =   0;
		graph_constraints.gridy   =   0;
		graph_constraints.gridwidth  = 1;
		graph_constraints.gridheight = 3;
		graph_constraints.fill   = GridBagConstraints.NONE;
		graph_constraints.anchor = GridBagConstraints.CENTER;

		npoints = 1000;
	
		trajectory.add(trajectory_options = new Panel());
		trajectory_options.setLayout(new GridLayout(2,2));
		trajectory_options.add(new Label(""));
		trajectory_options.add(var = new Choice());
		var.addItem("Phase");
		var.addItem("dPhase/dt");
		trajectory_options.add(new Label("Steps"));
		trajectory_options.add(time_tf = new TextField("1000",5));
		graph_layout.setConstraints(trajectory_options, graph_constraints);
	
		var.addItemListener(this);

		graph_constraints.weightx = 100;
		graph_constraints.weighty = 100;
		graph_constraints.gridx   =   1;
		graph_constraints.gridy   =   0;
		graph_constraints.gridwidth  = 2;
		graph_constraints.gridheight = 3;
		graph_constraints.fill   = GridBagConstraints.BOTH;

		trajectory.add(chart = new Chart(this));
		graph_layout.setConstraints(chart, graph_constraints);

		graph_constraints.weightx = 5;
		graph_constraints.weighty = 5;
		graph_constraints.gridx   =   0;
		graph_constraints.gridy   =   3;
		graph_constraints.gridwidth  = 3;
		graph_constraints.gridheight = 1;
		graph_constraints.fill   = GridBagConstraints.NONE;

		trajectory.add(start_button = new Button("Start"));
		graph_layout.setConstraints(start_button, graph_constraints);
		
		start_button.addActionListener(this);
		}

	public void actionPerformed(ActionEvent e)
		{
		if(e.getActionCommand().equals("Start"))
			{
			R = Integer.parseInt(R_tf.getText());
			C = Integer.parseInt(C_tf.getText());
			Ic = Integer.parseInt(Ic_tf.getText());
			I0 = Integer.parseInt(I0_tf.getText());
			I1 = Integer.parseInt(I1_tf.getText());
			omega1 = 2*Math.PI*Integer.parseInt(time_tf.getText());
			npoints = Integer.parseInt(time_tf.getText());

			omegac = _2esuhbar*R*Ic*1E-12;
			BetaC = _2esuhbar*R*R*C*Ic*1E-15;
			Omega1 = omega1 / omegac;

			chart.draw();
			}
		else if(e.getSource() instanceof Component && ((Component)e.getSource()).getParent().equals(tabs))
			((CardLayout)cards.getLayout()).show(cards,e.getActionCommand());
		}

	public void itemStateChanged(ItemEvent e)
		{
		if(e.getSource().equals(var))
			{
			if(e.getItem().equals("Phase"))
				chart.var = 0;
			else if(e.getItem().equals("dPhase/dt"))
				chart.var = 1;
			chart.repaint();
			}
		} 
	}

class Chart extends Canvas
	{
	int var;
	KillFrame parent;

	public Chart(KillFrame refparent)
		{
		var = 0;
		parent = refparent;
		}

	public void paint(Graphics g)
		{
		Dimension dim = getSize();
		g.drawRect(3,3,dim.width-6,dim.height-6);
		//g.drawLine(dim.width/2+3,3,dim.width/2+3,dim.height-6);
		g.drawLine(3,dim.height/2,dim.width-3,dim.height/2);
		}

	public void draw()
		{
		repaint();

		Integrate jj;
		jj = new Integrate(this,parent.npoints);

		jj.x = 0;
		jj.y[0] = jj.y[1] = 0;

        	jj.start();
		}

	public double f(double tx, double[] ty,int i)
		{
		double I0,I1,Omega1,BetaC;

		I0 = parent.I0;
		I1 = parent.I1;
		Omega1 = parent.Omega1;
		BetaC = parent.BetaC;

		switch(i)
			{
			case 0: return(ty[1]);
			case 1: return((I0+I1*Math.sin(Omega1*tx)-ty[1]-Math.sin(ty[0]))/BetaC);
			};
		return 0;
		}

	}

class JJS
	{
	public static void main(String[] args)
		{
		WindowListener l = new WindowAdapter()
			{
			public void windowClosing(WindowEvent e) {System.exit(0);}
			};		

		Frame f = new KillFrame();
		f.addWindowListener(l);
		f.setSize(600, 400);
		f.setBackground(Color.lightGray);
		f.show();
		}
	}
